@@ -1,5 +1,5 @@
 <?php
-/* SVN FILE: $Id: model_php5.php 5129 2007-05-20 05:47:57Z phpnut $ */
+/* SVN FILE: $Id: model_php5.php 5792 2007-10-19 04:33:24Z phpnut $ */
 /**
  * Object-relational mapper.
  *
@@ -21,9 +21,9 @@
  * @package			cake
  * @subpackage		cake.cake.libs.model
  * @since			CakePHP(tm) v 0.10.0.0
- * @version			$Revision: 5129 $
+ * @version			$Revision: 5792 $
  * @modifiedby		$LastChangedBy: phpnut $
- * @lastmodified	$Date: 2007-05-20 00:47:57 -0500 (Sun, 20 May 2007) $
+ * @lastmodified	$Date: 2007-10-18 23:33:24 -0500 (Thu, 18 Oct 2007) $
  * @license			http://www.opensource.org/licenses/mit-license.php The MIT License
  */
 /**
@@ -220,6 +220,18 @@ class Model extends Object{
  * @access public
  */
 	var $recursive = 1;
+/**
+ * Whitelist of fields allowed to be saved
+ *
+ * @var array
+ */
+	var $whitelist = array();
+/**
+ * Enter description here...
+ *
+ * @var boolean
+ */
+	var $cacheSources = true;
 /**
  * Default association keys
  *
@@ -289,9 +301,9 @@ class Model extends Object{
 
 		$this->id = $id;
 
-		if($table === false) {
+		if ($table === false) {
 			$this->useTable = false;
-		} else if($table) {
+		} elseif ($table) {
 			$this->useTable = $table;
 		}
 
@@ -347,10 +359,10 @@ class Model extends Object{
  */
 	function bindModel($params) {
 
-		foreach($params as $assoc => $model) {
+		foreach ($params as $assoc => $model) {
 			$this->__backAssociation[$assoc] = $this->{$assoc};
 
-			foreach($model as $key => $value) {
+			foreach ($model as $key => $value) {
 				$assocName = $key;
 
 				if (is_numeric($key)) {
@@ -372,10 +384,10 @@ class Model extends Object{
  * @access public
  */
 	function unbindModel($params) {
-		foreach($params as $assoc => $models) {
+		foreach ($params as $assoc => $models) {
 			$this->__backAssociation[$assoc] = $this->{$assoc};
 
-			foreach($models as $model) {
+			foreach ($models as $model) {
 				$this->__backAssociation = array_merge($this->__backAssociation, $this->{$assoc});
 				unset ($this->{$assoc}[$model]);
 			}
@@ -390,19 +402,19 @@ class Model extends Object{
 	function __createLinks() {
 
 		// Convert all string-based associations to array based
-		foreach($this->__associations as $type) {
+		foreach ($this->__associations as $type) {
 
 			if (!is_array($this->{$type})) {
 				$this->{$type} = explode(',', $this->{$type});
 
-				foreach($this->{$type} as $i => $className) {
+				foreach ($this->{$type} as $i => $className) {
 					$className = trim($className);
 					unset ($this->{$type}[$i]);
 					$this->{$type}[$className] = array();
 				}
 			}
 
-			foreach($this->{$type} as $assoc => $value) {
+			foreach ($this->{$type} as $assoc => $value) {
 				if (is_numeric($assoc)) {
 					unset ($this->{$type}[$assoc]);
 					$assoc = $value;
@@ -419,7 +431,7 @@ class Model extends Object{
 			}
 		}
 
-		foreach($this->__associations as $type) {
+		foreach ($this->__associations as $type) {
 			$this->__generateAssociation($type);
 		}
 	}
@@ -434,7 +446,7 @@ class Model extends Object{
 	function __constructLinkedModel($assoc, $className) {
 		$colKey = Inflector::underscore($className);
 
-		if(!class_exists($className)){
+		if (!class_exists($className)) {
 			loadModel($className);
 		}
 
@@ -457,10 +469,10 @@ class Model extends Object{
  * @access private
  */
 	function __generateAssociation($type) {
-		foreach($this->{$type}as $assocKey => $assocData) {
+		foreach ($this->{$type}as $assocKey => $assocData) {
 			$class = $assocKey;
 
-			foreach($this->__associationKeys[$type] as $key) {
+			foreach ($this->__associationKeys[$type] as $key) {
 				if (!isset($this->{$type}[$assocKey][$key]) || $this->{$type}[$assocKey][$key] == null) {
 					$data = '';
 
@@ -509,31 +521,23 @@ class Model extends Object{
  * @access public
  */
 	function setSource($tableName) {
+		$this->setDataSource($this->useDbConfig);
 		$db =& ConnectionManager::getDataSource($this->useDbConfig);
+		$db->cacheSources = $this->cacheSources;
 
 		if ($db->isInterfaceSupported('listSources')) {
-			$prefix = '';
-
-			if ($this->tablePrefix) {
-				$prefix = $this->tablePrefix;
-			}
-
 			$sources = $db->listSources();
-			if (is_array($sources) && !in_array(low($prefix . $tableName), array_map('low', $sources))) {
+			if (is_array($sources) && !in_array(low($this->tablePrefix . $tableName), array_map('low', $sources))) {
 				return $this->cakeError('missingTable', array(array(
 												'className' => $this->name,
-												'table' => $prefix . $tableName)));
-			} else {
-				$this->table = $tableName;
-				$this->tableToModel[$this->table] = $this->name;
-				$this->loadInfo();
-			}
+												'table' => $this->tablePrefix . $tableName)));
 
-		} else {
-			$this->table = $tableName;
-			$this->tableToModel[$this->table] = $this->name;
-			$this->loadInfo();
+			}
+			$this->_tableInfo = null;
 		}
+		$this->table = $this->useTable = $tableName;
+		$this->tableToModel[$this->table] = $this->name;
+		$this->loadInfo();
 	}
 /**
  * This function does two things: 1) it scans the array $one for the primary key,
@@ -559,21 +563,22 @@ class Model extends Object{
 			$data = array($this->name => array($one => $two));
 		}
 
-		foreach($data as $n => $v) {
+		foreach ($data as $n => $v) {
 			if (is_array($v)) {
 
-				foreach($v as $x => $y) {
-					if ($n == $this->name) {
+				foreach ($v as $x => $y) {
+					if (empty($this->whitelist) || (in_array($x, $this->whitelist) || $n !== $this->name)) {
 						if (isset($this->validationErrors[$x])) {
 							unset ($this->validationErrors[$x]);
 						}
 
-						if ($x === $this->primaryKey) {
-							$this->id = $y;
+						if ($n == $this->name || is_array($y)) {
+							if ($x === $this->primaryKey) {
+								$this->id = $y;
+							}
+							$this->data[$n][$x] = $y;
 						}
 					}
-
-					$this->data[$n][$x] = $y;
 				}
 			}
 		}
@@ -587,11 +592,18 @@ class Model extends Object{
  */
 	function loadInfo() {
 		$db =& ConnectionManager::getDataSource($this->useDbConfig);
+		$db->cacheSources = $this->cacheSources;
 
 		if (!is_object($this->_tableInfo) && $db->isInterfaceSupported('describe') && $this->useTable !== false) {
-			$this->_tableInfo = new Set($db->describe($this));
+			$info = new Set($db->describe($this));
+
+			foreach($info->value as $field => $value) {
+				$fields[] = am(array('name'=> $field), $value);
+			}
+			unset($info);
+			$this->_tableInfo = new Set($fields);
 		} elseif ($this->useTable === false) {
-			return new Set();
+			$this->_tableInfo = new Set();
 		}
 		return $this->_tableInfo;
 	}
@@ -607,7 +619,7 @@ class Model extends Object{
 		$db =& ConnectionManager::getDataSource($this->useDbConfig);
 		$cols = array();
 
-		foreach($columns as $col) {
+		foreach ($columns as $col) {
 			$cols[$col['name']] = $col['type'];
 		}
 		return $cols;
@@ -624,7 +636,7 @@ class Model extends Object{
 		$columns = $columns->value;
 		$cols = array();
 
-		foreach($columns as $col) {
+		foreach ($columns as $col) {
 			if ($col['name'] == $column) {
 				return $col['type'];
 			}
@@ -711,7 +723,6 @@ class Model extends Object{
 			return false;
 		}
 	}
-
 /**
  * Returns contents of a field in a query matching given conditions.
  *
@@ -722,7 +733,7 @@ class Model extends Object{
  * @access public
  */
 	function field($name, $conditions = null, $order = null) {
-		if ($conditions === null) {
+		if ($conditions === null && $this->id !== false) {
 			$conditions = array($this->name . '.' . $this->primaryKey => $this->id);
 		}
 
@@ -771,6 +782,13 @@ class Model extends Object{
  */
 	function save($data = null, $validate = true, $fieldList = array()) {
 		$db =& ConnectionManager::getDataSource($this->useDbConfig);
+		$_whitelist = $this->whitelist;
+
+		if (!empty($fieldList)) {
+			$this->whitelist = $fieldList;
+		} elseif ($fieldList === null) {
+			$this->whitelist = array();
+		}
 
 		if ($data) {
 			if (countdim($data) == 1) {
@@ -780,17 +798,16 @@ class Model extends Object{
 			}
 		}
 
-		$whitelist = !(empty($fieldList) || count($fieldList) == 0);
-
 		if ($validate && !$this->validates()) {
+			$this->whitelist = $_whitelist;
 			return false;
 		}
 
 		if (!$this->beforeSave()) {
+			$this->whitelist = $_whitelist;
 			return false;
 		}
 		$fields = $values = array();
-		$count = 0;
 
 		if (count($this->data) > 1) {
 			$weHaveMulti = true;
@@ -798,9 +815,10 @@ class Model extends Object{
 		} else {
 			$weHaveMulti = false;
 		}
+		$habtm = count($this->hasAndBelongsToMany);
 
-		foreach($this->data as $n => $v) {
-			if (isset($weHaveMulti) && isset($v[$n]) && $count > 0 && count($this->hasAndBelongsToMany) > 0) {
+		foreach ($this->data as $n => $v) {
+			if (isset($weHaveMulti) && isset($v[$n]) && $habtm > 0) {
 				$joined[] = $v;
 			} else {
 				if ($n === $this->name) {
@@ -810,29 +828,28 @@ class Model extends Object{
 						}
 					}
 
-					foreach($v as $x => $y) {
-						if ($this->hasField($x) && ($whitelist && in_array($x, $fieldList) || !$whitelist)) {
+					foreach ($v as $x => $y) {
+						if ($this->hasField($x)) {
 							$fields[] = $x;
 							$values[] = $y;
 						}
 					}
 				}
 			}
-			$count++;
 		}
 		$exists = $this->exists();
 
-		if (!$exists && $this->hasField('created') && !in_array('created', $fields) && ($whitelist && in_array('created', $fieldList) || !$whitelist)) {
+		if (!$exists && $this->hasField('created') && !in_array('created', $fields)) {
 			$fields[] = 'created';
 			$values[] = date('Y-m-d H:i:s');
 		}
 
-		if ($this->hasField('modified') && !in_array('modified', $fields) && ($whitelist && in_array('modified', $fieldList) || !$whitelist)) {
+		if ($this->hasField('modified') && !in_array('modified', $fields)) {
 			$fields[] = 'modified';
 			$values[] = date('Y-m-d H:i:s');
 		}
 
-		if ($this->hasField('updated') && !in_array('updated', $fields) && ($whitelist && in_array('updated', $fieldList) || !$whitelist)) {
+		if ($this->hasField('updated') && !in_array('updated', $fields)) {
 			$fields[] = 'updated';
 			$values[] = date('Y-m-d H:i:s');
 		}
@@ -840,6 +857,7 @@ class Model extends Object{
 		if (!$exists) {
 			$this->id = false;
 		}
+		$this->whitelist = $_whitelist;
 
 		if (count($fields)) {
 			if (!empty($this->id)) {
@@ -884,8 +902,8 @@ class Model extends Object{
  */
 	function __saveMulti($joined, $id) {
 		$db =& ConnectionManager::getDataSource($this->useDbConfig);
-		foreach($joined as $x => $y) {
-			foreach($y as $assoc => $value) {
+		foreach ($joined as $x => $y) {
+			foreach ($y as $assoc => $value) {
 				if (isset($this->hasAndBelongsToMany[$assoc])) {
 					$joinTable[$assoc] = $this->hasAndBelongsToMany[$assoc]['joinTable'];
 					$mainKey[$assoc] = $this->hasAndBelongsToMany[$assoc]['foreignKey'];
@@ -894,7 +912,7 @@ class Model extends Object{
 					$fields[$assoc]  = join(',', $keys);
 					unset($keys);
 
-					foreach($value as $update) {
+					foreach ($value as $update) {
 						if (!empty($update)) {
 							$values[]  = $db->value($id, $this->getColumnType($this->primaryKey));
 							$values[]  = $db->value($update);
@@ -917,7 +935,7 @@ class Model extends Object{
 		if (isset($joinTable)) {
 			$total = count($joinTable);
 
-			if(is_array($newValue)) {
+			if (is_array($newValue)) {
 				foreach ($newValue as $loopAssoc => $val) {
 					$db =& ConnectionManager::getDataSource($this->useDbConfig);
 					$table = $db->name($db->fullTableName($joinTable[$loopAssoc]));
@@ -925,7 +943,7 @@ class Model extends Object{
 
 					if (!empty($newValue[$loopAssoc])) {
 						$secondCount = count($newValue[$loopAssoc]);
-						for($x = 0; $x < $secondCount; $x++) {
+						for ($x = 0; $x < $secondCount; $x++) {
 							$db->query("INSERT INTO {$table} ({$fields[$loopAssoc]}) VALUES {$newValue[$loopAssoc][$x]}");
 						}
 					}
@@ -997,15 +1015,15 @@ class Model extends Object{
 			$savedAssociatons = $this->__backAssociation;
 			$this->__backAssociation = array();
 		}
-		foreach($this->hasMany as $assoc => $data) {
+		foreach ($this->hasMany as $assoc => $data) {
 			if ($data['dependent'] === true && $cascade === true) {
 				$model =& $this->{$data['className']};
 				$field = $model->escapeField($data['foreignKey']);
 				$model->recursive = 0;
 				$records = $model->findAll("$field = '$id'", $model->primaryKey, null, null);
 
-				if($records != false){
-					foreach($records as $record) {
+				if ($records != false) {
+					foreach ($records as $record) {
 						$model->del($record[$data['className']][$model->primaryKey]);
 					}
 				}
@@ -1027,15 +1045,15 @@ class Model extends Object{
 			$savedAssociatons = $this->__backAssociation;
 			$this->__backAssociation = array();
 		}
-		foreach($this->hasOne as $assoc => $data) {
+		foreach ($this->hasOne as $assoc => $data) {
 			if ($data['dependent'] === true && $cascade === true) {
 				$model =& $this->{$data['className']};
 				$field = $model->escapeField($data['foreignKey']);
 				$model->recursive = 0;
 				$records = $model->findAll("$field = '$id'", $model->primaryKey, null, null);
 
-				if($records != false){
-					foreach($records as $record) {
+				if ($records != false) {
+					foreach ($records as $record) {
 						$model->del($record[$data['className']][$model->primaryKey]);
 					}
 				}
@@ -1054,7 +1072,7 @@ class Model extends Object{
  */
 	function _deleteMulti($id) {
 		$db =& ConnectionManager::getDataSource($this->useDbConfig);
-		foreach($this->hasAndBelongsToMany as $assoc => $data) {
+		foreach ($this->hasAndBelongsToMany as $assoc => $data) {
 			$db->execute("DELETE FROM " . $db->name($db->fullTableName($data['joinTable'])) . " WHERE " . $db->name($data['foreignKey']) . " = '{$id}'");
 		}
 	}
@@ -1170,7 +1188,7 @@ class Model extends Object{
  * @access private
  */
 	function __resetAssociations() {
-		foreach($this->__associations as $type) {
+		foreach ($this->__associations as $type) {
 			if (isset($this->__backAssociation[$type])) {
 				$this->{$type} = $this->__backAssociation[$type];
 			}
@@ -1190,8 +1208,8 @@ class Model extends Object{
 		$db =& ConnectionManager::getDataSource($this->useDbConfig);
 		$data = $db->fetchAll($data, $this->cacheQueries);
 
-		foreach($data as $key => $value) {
-			foreach($this->tableToModel as $key1 => $value1) {
+		foreach ($data as $key => $value) {
+			foreach ($this->tableToModel as $key1 => $value1) {
 				if (isset($data[$key][$key1])) {
 					$newData[$key][$value1] = $data[$key][$key1];
 				}
@@ -1214,7 +1232,9 @@ class Model extends Object{
  * @access public
  */
 	function findCount($conditions = null, $recursive = 0) {
-		list($data) = $this->findAll($conditions, 'COUNT(*) AS count', null, null, 1, $recursive);
+		$db =& ConnectionManager::getDataSource($this->useDbConfig);
+
+		list($data) = $this->findAll($conditions, 'COUNT(*) AS ' . $db->name('count'), null, null, 1, $recursive);
 
 		if (isset($data[0]['count'])) {
 			return $data[0]['count'];
@@ -1251,7 +1271,7 @@ class Model extends Object{
 		$out = array();
 		$sizeOf = sizeof($data);
 
-		for($ii = 0; $ii < $sizeOf; $ii++) {
+		for ($ii = 0; $ii < $sizeOf; $ii++) {
 			if (($data[$ii][$this->name]['parent_id'] == $root) || (($root === null) && ($data[$ii][$this->name]['parent_id'] == '0'))) {
 				$tmp = $data[$ii];
 
@@ -1350,7 +1370,7 @@ class Model extends Object{
 			$data = $data[$this->name];
 		}
 
-		foreach($this->validate as $field_name => $validator) {
+		foreach ($this->validate as $field_name => $validator) {
 			if (isset($data[$field_name]) && !preg_match($validator, $data[$field_name])) {
 				$this->invalidate($field_name);
 			}
@@ -1381,7 +1401,7 @@ class Model extends Object{
 		$foreignKeys = array();
 
 		if (count($this->belongsTo)) {
-			foreach($this->belongsTo as $assoc => $data) {
+			foreach ($this->belongsTo as $assoc => $data) {
 				$foreignKeys[] = $data['foreignKey'];
 			}
 		}
@@ -1416,13 +1436,13 @@ class Model extends Object{
 		}
 		$recursive = $this->recursive;
 
-		if($recursive >= 1) {
+		if ($recursive >= 1) {
 			$this->recursive = -1;
 		}
 		$result = $this->findAll($conditions, $fields, $order, $limit);
 		$this->recursive = $recursive;
 
-		if(!$result) {
+		if (!$result) {
 			return false;
 		}
 
@@ -1474,7 +1494,7 @@ class Model extends Object{
 			return $this->id[$list];
 		}
 
-		foreach($this->id as $id) {
+		foreach ($this->id as $id) {
 			return $id;
 		}
 
@@ -1535,13 +1555,12 @@ class Model extends Object{
  * @access public
  */
 	function setDataSource($dataSource = null) {
-		if ($dataSource == null) {
-			$dataSource = $this->useDbConfig;
+		if ($dataSource != null) {
+			$this->useDbConfig = $dataSource;
 		}
+		$db =& ConnectionManager::getDataSource($this->useDbConfig);
 
-		$db =& ConnectionManager::getDataSource($dataSource);
-
-		if (!empty($db->config['prefix']) && $this->tablePrefix == null) {
+		if (!empty($db->config['prefix']) && $this->tablePrefix === null) {
 			$this->tablePrefix = $db->config['prefix'];
 		}
 
@@ -1614,6 +1633,13 @@ class Model extends Object{
 	function beforeValidate() {
 		return true;
 	}
+/**
+ * DataSource error callback
+ *
+ * @return void
+ */
+	function onError() {
+	}
 /**
  * Private method.  Clears cache for this model
  *
@@ -1627,8 +1653,8 @@ class Model extends Object{
 			if (defined('CACHE_CHECK') && CACHE_CHECK === true) {
 				$assoc[] = strtolower(Inflector::pluralize($this->name));
 
-				foreach($this->__associations as $key => $association) {
-					foreach($this->$association as $key => $className) {
+				foreach ($this->__associations as $key => $association) {
+					foreach ($this->$association as $key => $className) {
 						$check = strtolower(Inflector::pluralize($className['className']));
 
 						if (!in_array($check, $assoc)) {
