@@ -1,5 +1,5 @@
 <?php
-/* SVN FILE: $Id: session.php 5129 2007-05-20 05:47:57Z phpnut $ */
+/* SVN FILE: $Id: session.php 5792 2007-10-19 04:33:24Z phpnut $ */
 /**
  * Session class for Cake.
  *
@@ -24,9 +24,9 @@
  * @package			cake
  * @subpackage		cake.cake.libs
  * @since			CakePHP(tm) v .0.10.0.1222
- * @version			$Revision: 5129 $
+ * @version			$Revision: 5792 $
  * @modifiedby		$LastChangedBy: phpnut $
- * @lastmodified	$Date: 2007-05-20 00:47:57 -0500 (Sun, 20 May 2007) $
+ * @lastmodified	$Date: 2007-10-18 23:33:24 -0500 (Thu, 18 Oct 2007) $
  * @license			http://www.opensource.org/licenses/mit-license.php The MIT License
  */
 /**
@@ -71,7 +71,7 @@ class CakeSession extends Object {
  * @var string
  * @access protected
  */
-	var $_userAgent = false;
+	var $_userAgent = '';
 /**
  * Path to where the session is active.
  *
@@ -118,21 +118,21 @@ class CakeSession extends Object {
  * Constructor.
  *
  * @param string $base The base path for the Session
- * @param boolean $start
+ * @param boolean $start Should session be started right now
  * @access public
  */
 	function __construct($base = null, $start = true) {
-		if (env('HTTP_USER_AGENT') != null) {
-			$this->_userAgent = md5(env('HTTP_USER_AGENT') . CAKE_SESSION_STRING);
-		} else {
-				$this->_userAgent = "";
+		if (Configure::read('Session.checkAgent') === true) {
+			if (env('HTTP_USER_AGENT') != null) {
+				$this->_userAgent = md5(env('HTTP_USER_AGENT') . CAKE_SESSION_STRING);
+			}
 		}
 		$this->time = time();
 
-		if($start === true) {
+		if ($start === true) {
 			$this->host = env('HTTP_HOST');
 
-			if (empty($base) || strpos($base, '?')) {
+			if (empty($base) || strpos($base, '?' || strpos($base, 'index.php'))) {
 				$this->path = '/';
 			} else {
 				$this->path = $base;
@@ -150,9 +150,7 @@ class CakeSession extends Object {
 			}
 
 			$this->__initSession();
-			session_cache_limiter ("must-revalidate");
-			session_start();
-			header ('P3P: CP="NOI ADM DEV PSAi COM NAV OUR OTRo STP IND DEM"');
+			$this->__startSession();
 			$this->__checkValid();
 		}
 		parent::__construct();
@@ -174,17 +172,21 @@ class CakeSession extends Object {
 	}
 
 /**
- * Temp method until we are able to remove the last eval()
+ * Temp method until we are able to remove the last eval().
+ * Builds an expression to fetch a session variable with specified name.
+ *
+ * @param string $name Name of variable (in dot notation)
+ * @access private
  */
 	function __sessionVarNames($name) {
-		if (is_string($name) && preg_match("/^[0-9a-zA-Z._-]+$/", $name)) {
+		if (is_string($name) && preg_match("/^[ 0-9a-zA-Z._-]*$/", $name)) {
 			if (strpos($name, ".")) {
 				$names = explode(".", $name);
 			} else {
 				$names = array($name);
 			}
-			$expression="\$_SESSION";
-			foreach($names as $item) {
+			$expression = "\$_SESSION";
+			foreach ($names as $item) {
 				$expression .= is_numeric($item) ? "[$item]" : "['$item']";
 			}
 			return $expression;
@@ -215,9 +217,8 @@ class CakeSession extends Object {
 /**
  * Used to write new data to _SESSION, since PHP doesn't like us setting the _SESSION var itself
  *
- * @param array $old
- * @param array $new
- * @return void
+ * @param array $old Set of old variables => values
+ * @param array $new New set of variable => value
  * @access private
  */
 	function __overwrite(&$old, $new) {
@@ -233,9 +234,9 @@ class CakeSession extends Object {
 /**
  * Return error description for given error number.
  *
- * @param int $errorNumber
+ * @param int $errorNumber Error to set
  * @return string Error as string
- * @access public
+ * @access private
  */
 	function __error($errorNumber) {
 		if (!is_array($this->error) || !array_key_exists($errorNumber, $this->error)) {
@@ -260,13 +261,15 @@ class CakeSession extends Object {
 /**
  * Returns true if session is valid.
  *
- * @return boolean
+ * @return boolean Success
  * @access public
  */
 	function valid() {
 		if ($this->read('Config')) {
-			if ($this->_userAgent == $this->read("Config.userAgent") && $this->time <= $this->read("Config.time")) {
-				$this->valid = true;
+			if (Configure::read('Session.checkAgent') === false || $this->_userAgent == $this->read("Config.userAgent") && $this->time <= $this->read("Config.time")) {
+				if ($this->error === false) {
+					$this->valid = true;
+				}
 			} else {
 				$this->valid = false;
 				$this->__setError(1, "Session Highjacking Attempted !!!");
@@ -277,7 +280,7 @@ class CakeSession extends Object {
 /**
  * Returns given session variable, or all of them, if no parameters given.
  *
- * @param mixed $name The name of the session variable
+ * @param mixed $name The name of the session variable (or a path as sent to Set.extract)
  * @return mixed The value of the session variable
  * @access public
  */
@@ -300,7 +303,7 @@ class CakeSession extends Object {
  * Returns all session variables.
  *
  * @return mixed Full $_SESSION array, or false on error.
- * @access public
+ * @access private
  */
 	function __returnSessionVars() {
 		if (!empty($_SESSION)) {
@@ -313,7 +316,7 @@ class CakeSession extends Object {
  * Tells Session to write a notification when a certain session path or subpath is written to
  *
  * @param mixed $var The variable path to watch
- * @return void
+ * @access public
  */
 	function watch($var) {
 		$var = $this->__validateKeys($var);
@@ -326,7 +329,7 @@ class CakeSession extends Object {
  * Tells Session to stop watching a given key path
  *
  * @param mixed $var The variable path to watch
- * @return void
+ * @access public
  */
 	function ignore($var) {
 		$var = $this->__validateKeys($var);
@@ -344,9 +347,10 @@ class CakeSession extends Object {
 /**
  * Writes value to given session variable name.
  *
- * @param mixed $name
- * @param string $value
+ * @param mixed $name Name of variable
+ * @param string $value Value to write
  * @return boolean True if the write was successful, false if the write failed
+ * @access public
  */
 	function write($name, $value) {
 		$var = $this->__validateKeys($name);
@@ -355,16 +359,15 @@ class CakeSession extends Object {
 			return false;
 		}
 		if (in_array($var, $this->watchKeys)) {
-			trigger_error('Writing session key {' . $var . '}: ' . Debugger::exportVar($value), E_USER_NOTICE);
+			trigger_error('Writing session key {' . $var . '}: ' . print_r($value), E_USER_NOTICE);
 		}
 		$this->__overwrite($_SESSION, Set::insert($_SESSION, $var, $value));
-		return (Set::extract($_SESSION, $var) == $value);
+		return (Set::extract($_SESSION, $var) === $value);
 	}
 /**
  * Helper method to destroy invalid sessions.
  *
- * @return void
- * @access private
+ * @access public
  */
 	function destroy() {
 		$sessionpath = session_save_path();
@@ -386,7 +389,6 @@ class CakeSession extends Object {
 /**
  * Helper method to initialize a session, based on Cake core settings.
  *
- * @return void
  * @access private
  */
 	function __initSession() {
@@ -399,6 +401,9 @@ class CakeSession extends Object {
 			break;
 			case 'medium':
 				$this->cookieLifeTime = 7 * 86400;
+				if (function_exists('ini_set')) {
+					ini_set('session.referer_check', $this->host);
+				}
 			break;
 			case 'low':
 			default:
@@ -464,22 +469,39 @@ class CakeSession extends Object {
 			break;
 		}
 	}
+/**
+ * Helper method to start a session
+ *
+ * @access private
+ */
+	function __startSession() {
+		if (headers_sent()) {
+			if (!isset($_SESSION)) {
+				$_SESSION = array();
+			}
+		} else {
+			session_cache_limiter ("must-revalidate");
+			session_start();
+			if (CAKE_SECURITY === 'high') {
+				$this->renew();
+			}
+			header ('P3P: CP="NOI ADM DEV PSAi COM NAV OUR OTRo STP IND DEM"');
+		}
+	}
 /**
  * Helper method to create a new session.
  *
- * @return void
  * @access private
- *
  */
 	function __checkValid() {
 		if ($this->read('Config')) {
-			if ($this->_userAgent == $this->read("Config.userAgent") && $this->time <= $this->read("Config.time")) {
+			if (Configure::read('Session.checkAgent') === false || $this->_userAgent == $this->read("Config.userAgent") && $this->time <= $this->read("Config.time")) {
 				$this->write("Config.time", $this->sessionTime);
 				$this->valid = true;
 			} else {
+				$this->destroy();
 				$this->valid = false;
 				$this->__setError(1, "Session Highjacking Attempted !!!");
-				$this->destroy();
 			}
 		} else {
 			srand ((double)microtime() * 1000000);
@@ -493,7 +515,6 @@ class CakeSession extends Object {
 /**
  * Helper method to restart a session.
  *
- * @return void
  * @access private
  */
 	function __regenerateId() {
@@ -525,7 +546,6 @@ class CakeSession extends Object {
 /**
  * Restarts this session.
  *
- * @return void
  * @access public
  */
 	function renew() {
@@ -536,11 +556,11 @@ class CakeSession extends Object {
  * example: $name = 'ControllerName.key';
  *
  * @param string $name Session key names as string.
- * @return boolean false is $name is not correct format
+ * @return mixed false is $name is not correct format, or $name if it is correct
  * @access private
  */
 	function __validateKeys($name) {
-		if (is_string($name) && preg_match("/^[0-9a-zA-Z._-]+$/", $name)) {
+		if (is_string($name) && preg_match("/^[ 0-9a-zA-Z._-]*$/", $name)) {
 			return $name;
 		}
 		$this->__setError(3, "$name is not a string");
@@ -551,7 +571,6 @@ class CakeSession extends Object {
  *
  * @param int $errorNumber Number of the error
  * @param string $errorMessage Description of the error
- * @return void
  * @access private
  */
 	function __setError($errorNumber, $errorMessage) {
@@ -562,33 +581,29 @@ class CakeSession extends Object {
 		$this->lastError = $errorNumber;
 	}
 /**
- * Method called on open of a database
- * sesson
+ * Method called on open of a database session.
  *
- * @return boolean
+ * @return boolean Success
  * @access private
- *
  */
 	function __open() {
 		return true;
 	}
 /**
- * Method called on close of a database
- * session
+ * Method called on close of a database session.
  *
- * @return boolean
+ * @return boolean Success
  * @access private
  */
 	function __close() {
 		$probability = mt_rand(1, 150);
-		if($probability <= 3) {
+		if ($probability <= 3) {
 			CakeSession::__gc();
 		}
 		return true;
 	}
 /**
- * Method used to read from a database
- * session
+ * Method used to read from a database session.
  *
  * @param mixed $key The key of the value to read
  * @return mixed The value of the key or false if it does not exist
@@ -610,12 +625,11 @@ class CakeSession extends Object {
 		}
 	}
 /**
- * Helper function called on write for database
- * sessions
+ * Helper function called on write for database sessions.
  *
  * @param mixed $key The name of the var
  * @param mixed $value The value of the var
- * @return boolean
+ * @return boolean Success
  * @access private
  */
 	function __write($key, $value) {
@@ -655,11 +669,10 @@ class CakeSession extends Object {
 		return true;
 	}
 /**
- * Method called on the destruction of a
- * database session
+ * Method called on the destruction of a database session.
  *
- * @param integer $key
- * @return boolean
+ * @param int $key Key that uniquely identifies session in database
+ * @return boolean Success
  * @access private
  */
 	function __destroy($key) {
@@ -669,11 +682,10 @@ class CakeSession extends Object {
 		return true;
 	}
 /**
- * Helper function called on gc for
- * database sessions
+ * Helper function called on gc for database sessions.
  *
- * @param unknown_type $expires
- * @return boolean
+ * @param int $expires Timestamp (defaults to current time)
+ * @return boolean Success
  * @access private
  */
 	function __gc($expires = null) {
